@@ -178,6 +178,7 @@ void main_entry(int argc, char *argv[])
       strcat(definestr,")");
       eq = strchr(definestr,'=');
       *eq = ' ';
+      printf("command-line param: %s\n", argv[i]);
       gh_eval_str(definestr);
       { /* add the name of the defined variable to params-set-list */
 	   char *remember_define;
@@ -211,6 +212,7 @@ void main_entry(int argc, char *argv[])
        gh_repl(argc - i, argv + i); /* skip already-handled args */
 
  done:
+  ;
 #ifdef HAVE_CTL_HOOKS
   /* Note that the stop hook will never be called if we are in
      interactive mode, because gh_repl calls exit().  Oh well. */
