@@ -52,6 +52,8 @@
 ;    find-root: find the root of a function of one argument
 ; All routines use quadratically convergent methods.
 
+; ****************************************************************
+
 (define min-arg car)
 (define min-val cdr)
 (define max-arg min-arg)
@@ -176,6 +178,8 @@
 		  (sixth bracket)
 		  0 0))))))))
 
+; ****************************************************************
+
 ; (minimize-multiple f tol arg1 arg2 ... argN) :
 ;      Minimize a function f of N arguments, given the fractional tolerance
 ; desired and initial guesses for the arguments.
@@ -280,6 +284,7 @@
 		       (append (list (compose - f) tol) guess-args))))
     (cons (min-arg result) (- (min-val result)))))
 
+; ****************************************************************
 ; Find a root of a function of one argument using Ridder's method.
 
 ; (find-root f tol x-min x-max) : returns the root of the function (f x),
@@ -320,6 +325,52 @@
 	
   (ridder x-min x-max (f x-min) (f x-max)))
 
+; ****************************************************************
+; Find a root by Newton's method with bounds and bisection,
+; given a function f that returns a pair of (value . derivative)
+
+(define (find-root-deriv f tol x-min x-max . x-guess)
+  ; Some trickiness: we only need to evaluate the function at x-min and
+  ; x-max if a Newton step fails, and even then only if we haven't already
+  ; bracketed the root, so do this via lazy evaluation.
+  (define f-memo (memoize f))
+  (define (lazy x) (if (number? x) x (x)))
+  (define ((pick-bound which?))
+    (let ((fmin-pair (f-memo x-min)) (fmax-pair (f-memo x-max)))
+      (let ((fmin (car fmin-pair)) (fmax (car fmax-pair)))
+	(if (which? fmin) x-min
+	    (if (which? fmax) x-max
+		(error "failed to bracket the root in find-root-deriv"))))))
+
+  (define (in-bounds? x f df a b)
+    (negative? (* (- f (* df (- x a)))
+		  (- f (* df (- x b))))))
+	  
+  (define (newton x a b dx)
+    (if (< (abs dx) (abs (* tol x)))
+	x
+	(let ((fx-pair (f-memo x)))
+	  (let ((f (car fx-pair)) (df (cdr fx-pair)))
+	    (if (= f 0)
+		x
+		(let ((a' (if (< f 0) x a)) (b' (if (> f 0) x b)))
+		  (if (and (if (and (number? a) (number? b))
+			       (in-bounds? x f df a b)
+			       (in-bounds? x f df x-min x-max))
+;			   (> (abs (* 0.5 dx df)) (abs f))
+			   )
+		      (newton (- x (/ f df)) a' b' (/ f df))
+		      (let ((av (lazy a)) (bv (lazy b)))
+			(let ((dx' (* 0.5 (- bv av)))
+			      (a'' (if (eq? a a') av a'))
+			      (b'' (if (eq? b b') bv b')))
+			  (newton (* (+ av bv) 0.5) a'' b'' dx'))))))))))
+
+  (newton (if (null? x-guess) (* (+ x-min x-max) 0.5) (car x-guess))
+	  (pick-bound negative?)
+	  (pick-bound positive?)
+	  (- x-max x-min)))
+
 ; ****************************************************************
 
 ; Numerical differentiation:
@@ -336,25 +387,27 @@
   (define (deriv-a a0 prev-a fac fac0)
     (if (null? prev-a)
 	(list a0)
-	(cons a0 (deriv-a (/ (- (* a0 fac) (car prev-a)) (- fac 1))
+	(cons a0 (deriv-a (binary/
+			   (binary- (binary* a0 fac) (car prev-a)) 
+			   (- fac 1))
 			  (cdr prev-a) (* fac fac0) fac0))))
   
   (define (deriv dx df0 err0 prev-a fac0)
-    (let ((a (deriv-a (/ (- (f (+ x dx)) (f (- x dx))) (* 2 dx))
+    (let ((a (deriv-a (binary/ (binary- (f (+ x dx)) (f (- x dx))) (* 2 dx))
 		      prev-a fac0 fac0)))
       (if (null? prev-a)
 	  (deriv (/ dx (sqrt fac0)) (car a) err0 a fac0)
 	  (let* ((errs
 		  (map max
-		       (map magnitude (map - (cdr a) (reverse (cdr (reverse a)))))
-		       (map magnitude (map - (cdr a) prev-a))))
+		       (map unary-abs (map binary- (cdr a) (reverse (cdr (reverse a)))))
+		       (map unary-abs (map binary- (cdr a) prev-a))))
 		 (errmin (apply min errs))
 		 (err (min errmin err0))
 		 (df (if (> err err0)
 			 df0
 			 (cdr (assoc errmin (map cons errs (cdr a)))))))
-	    (if (or (< err (* tol df)) 
-		    (> (magnitude (- (car (reverse a)) (car (reverse prev-a))))
+	    (if (or (< err (* tol (unary-abs df)) )
+		    (> (unary-abs (binary- (car (reverse a)) (car (reverse prev-a))))
 		       (* 2 err)))
 		(list df err)
 		(deriv (/ dx (sqrt fac0)) df err a fac0))))))
@@ -383,19 +436,70 @@
 ; Compute both the first and second derivatives at the same time
 ; (using minimal extra function evaluations).
 (define (derivative2 f x . dx-and-tol)
-  (define f-memo-tab '())
-  (define (f-memo y)
-    (let ((tab-val (assoc y f-memo-tab)))
-      (if tab-val
-	  (cdr tab-val)
-	  (let ((fy (f y)))
-	    (set! f-memo-tab (cons (cons y fy) f-memo-tab))
-	    fy))))
+  (define f-memo (memoize f))
   (define (f-deriv y)
-    (* 2 (/ (- (f-memo y) (f-memo x))
-	    (- y x))))
+    (binary* (binary- (f-memo y) (f-memo x)) (/ 2 (- y x))))
   (append
    (do-derivative-wrap f-memo x dx-and-tol)
    (do-derivative-wrap f-deriv x dx-and-tol)))
 
+(define (deriv2 f x . dx-and-tol)
+  (derivative-d2f (apply derivative2 (cons f (cons x dx-and-tol)))))
+
+; ****************************************************************
+
+; Some simple integration routines using an adaptive trapezoidal rule
+; (see e.g. Numerical Recipes, Sec. 4.2).  It might be nice to have
+; Gaussian quadratures and what-not, but on the other hand the
+; functions we are integrating may well be the result of a computation
+; on a finite grid (somehow interpolated), and so will not be smooth.
+; Also, implementing thse simple algorithms in Scheme lets us use our
+; polymorphic arithmetic functions so that we can easily integrate
+; real, complex, and vector-valued functions.
+
+; Integrate the 1d function (f x) from x=a..b to within the specified
+; fractional tolerance.
+(define (integrate-1d f a b tol)
+  (define (trap0 n sum)
+    (binary*
+     0.5
+     (binary+
+      sum
+      (if (<= n 1)
+	  (binary* (- b a) (binary+ (f a) (f b)))
+	  (let ((steps (pow2 (- n 2))))
+	    (let ((dx (/ (- b a) steps)))
+	      (binary* 
+	       dx
+	       (do ((cur-sum 0) (i 0 (+ i 1)) (x (+ a dx) (+ x dx)))
+		   ((>= i steps) cur-sum)
+		 (set! cur-sum (binary+ cur-sum (f x)))))))))))
+  (define (trap n sum)
+    (let ((newsum (trap0 n sum)))
+      (if (and (> n 5)
+	       (or (> n 20) 
+		   (binary= newsum sum)
+		    (< (unary-abs (binary- newsum sum))
+		       (* tol (unary-abs newsum)))))
+	  newsum
+	  (trap (+ n 1) newsum))))
+  (trap 1 0.0))
+	  
+; Integrate the multi-dimensional function f from a..b, within the
+; specified tolerance.  a and b are either numbers (for 1d integrals),
+; or vectors/lists of the same length giving the bounds in each dimension.
+(define (integrate f a b tol)
+  (define (int f a b)
+    (if (null? a)
+	(f)
+	(integrate-1d
+	 (lambda (x) (int (lambda (. y) (apply f (cons x y))) (cdr a) (cdr b)))
+	 (car a) (car b) tol)))
+  (cond
+   ((and (vector? a) (vector? b))
+    (integrate f (vector->list a) (vector->list b) tol))
+   ((and (number? a) (number? b))
+    (integrate f (list a) (list b) tol))
+   (else (int f a b))))
+
 ; ****************************************************************
